#! /usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bindings_factory_1 = require("@comunica/bindings-factory");
const sparqlalgebrajs_1 = require("sparqlalgebrajs");
const SyncEvaluator_1 = require("../lib/evaluators/SyncEvaluator");
const USAGE = `
Usage: sparqlee <expression>
Example: sparqlee 'concat("foo", "bar")'
`;
function template(expr) {
    return `
PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
PREFIX fn: <https://www.w3.org/TR/xpath-functions#>
PREFIX err: <http://www.w3.org/2005/xqt-errors#>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>

SELECT * WHERE { ?s ?p ?o FILTER (${expr})}
`;
}
function parse(query) {
    const sparqlQuery = (0, sparqlalgebrajs_1.translate)(query);
    // Extract filter expression from complete query
    return sparqlQuery.input.expression;
}
async function main() {
    if (process.argv.length < 3) {
        // eslint-disable-next-line no-console
        console.log(USAGE);
        return;
    }
    const input = process.argv[2];
    const expression = parse(template(input));
    const evaluator = new SyncEvaluator_1.SyncEvaluator(expression);
    const result = evaluator.evaluate(new bindings_factory_1.BindingsFactory().bindings());
    // eslint-disable-next-line no-console
    console.log(result);
}
main().catch(error => {
    // eslint-disable-next-line no-console
    console.log(error);
});
//# sourceMappingURL=sparqlee.js.map