export declare type KnownLiteralTypes = TypeAlias | TypeURL;
export declare enum TypeAlias {
    SPARQL_NUMERIC = "SPARQL_NUMERIC",
    /**
     * Stringly is everything defined in https://www.w3.org/TR/sparql11-query/#func-strings
     * In other words it is a simple literal, a plain literal with language tag, or a literal with datatype xsd:string
     * In other words, since sparqlee transforms a simple literal to xsd_string. It is RDF_LANG_STRING or XSD_STRING.
     * Reasons for this are mentioned here: w3c/sparql-12#112
     */
    SPARQL_STRINGLY = "SPARQL_STRINGLY",
    SPARQL_NON_LEXICAL = "SPARQL_NON_LEXICAL"
}
export declare enum TypeURL {
    XSD_ANY_URI = "http://www.w3.org/2001/XMLSchema#anyURI",
    XSD_STRING = "http://www.w3.org/2001/XMLSchema#string",
    RDF_LANG_STRING = "http://www.w3.org/1999/02/22-rdf-syntax-ns#langString",
    XSD_BOOLEAN = "http://www.w3.org/2001/XMLSchema#boolean",
    XSD_DATE_TIME = "http://www.w3.org/2001/XMLSchema#dateTime",
    XSD_DATE_TIME_STAMP = "http://www.w3.org/2001/XMLSchema#dateTimeStamp",
    XSD_DATE = "http://www.w3.org/2001/XMLSchema#date",
    XSD_G_MONTH = "http://www.w3.org/2001/XMLSchema#gMonth",
    XSD_G_MONTHDAY = "http://www.w3.org/2001/XMLSchema#gMonthDay",
    XSD_G_YEAR = "http://www.w3.org/2001/XMLSchema#gYear",
    XSD_G_YEAR_MONTH = "http://www.w3.org/2001/XMLSchema#gYearMonth",
    XSD_TIME = "http://www.w3.org/2001/XMLSchema#time",
    XSD_G_DAY = "http://www.w3.org/2001/XMLSchema#gDay",
    XSD_DECIMAL = "http://www.w3.org/2001/XMLSchema#decimal",
    XSD_FLOAT = "http://www.w3.org/2001/XMLSchema#float",
    XSD_DOUBLE = "http://www.w3.org/2001/XMLSchema#double",
    XSD_INTEGER = "http://www.w3.org/2001/XMLSchema#integer",
    XSD_NON_POSITIVE_INTEGER = "http://www.w3.org/2001/XMLSchema#nonPositiveInteger",
    XSD_NEGATIVE_INTEGER = "http://www.w3.org/2001/XMLSchema#negativeInteger",
    XSD_LONG = "http://www.w3.org/2001/XMLSchema#long",
    XSD_INT = "http://www.w3.org/2001/XMLSchema#int",
    XSD_SHORT = "http://www.w3.org/2001/XMLSchema#short",
    XSD_BYTE = "http://www.w3.org/2001/XMLSchema#byte",
    XSD_NON_NEGATIVE_INTEGER = "http://www.w3.org/2001/XMLSchema#nonNegativeInteger",
    XSD_POSITIVE_INTEGER = "http://www.w3.org/2001/XMLSchema#positiveInteger",
    XSD_UNSIGNED_LONG = "http://www.w3.org/2001/XMLSchema#unsignedLong",
    XSD_UNSIGNED_INT = "http://www.w3.org/2001/XMLSchema#unsignedInt",
    XSD_UNSIGNED_SHORT = "http://www.w3.org/2001/XMLSchema#unsignedShort",
    XSD_UNSIGNED_BYTE = "http://www.w3.org/2001/XMLSchema#unsignedByte",
    XSD_NORMALIZED_STRING = "http://www.w3.org/2001/XMLSchema#normalizedString",
    XSD_TOKEN = "http://www.w3.org/2001/XMLSchema#token",
    XSD_LANGUAGE = "http://www.w3.org/2001/XMLSchema#language",
    XSD_NM_TOKEN = "http://www.w3.org/2001/XMLSchema#NMTOKEN",
    XSD_NAME = "http://www.w3.org/2001/XMLSchema#name",
    XSD_NC_NAME = "http://www.w3.org/2001/XMLSchema#NCName",
    XSD_ENTITY = "http://www.w3.org/2001/XMLSchema#ENTITY",
    XSD_ID = "http://www.w3.org/2001/XMLSchema#ID",
    XSD_ID_REF = "http://www.w3.org/2001/XMLSchema#IDREF",
    XSD_DURATION = "http://www.w3.org/2001/XMLSchema#duration",
    XSD_YEAR_MONTH_DURATION = "http://www.w3.org/2001/XMLSchema#yearMonthDuration",
    XSD_DAY_TIME_DURATION = "http://www.w3.org/2001/XMLSchema#dayTimeDuration"
}
export declare type Operator = RegularOperator | SpecialOperator;
export declare enum RegularOperator {
    NOT = "!",
    UMINUS = "uminus",
    UPLUS = "uplus",
    EQUAL = "=",
    NOT_EQUAL = "!=",
    LT = "<",
    GT = ">",
    LTE = "<=",
    GTE = ">=",
    MULTIPLICATION = "*",
    DIVISION = "/",
    ADDITION = "+",
    SUBTRACTION = "-",
    IS_IRI = "isiri",
    IS_URI = "isuri",
    IS_BLANK = "isblank",
    IS_LITERAL = "isliteral",
    IS_NUMERIC = "isnumeric",
    STR = "str",
    LANG = "lang",
    DATATYPE = "datatype",
    IRI = "iri",
    URI = "uri",
    STRDT = "strdt",
    STRLANG = "strlang",
    UUID = "uuid",
    STRUUID = "struuid",
    STRLEN = "strlen",
    SUBSTR = "substr",
    UCASE = "ucase",
    LCASE = "lcase",
    STRSTARTS = "strstarts",
    STRENDS = "strends",
    CONTAINS = "contains",
    STRBEFORE = "strbefore",
    STRAFTER = "strafter",
    ENCODE_FOR_URI = "encode_for_uri",
    LANG_MATCHES = "langmatches",
    REGEX = "regex",
    REPLACE = "replace",
    ABS = "abs",
    ROUND = "round",
    CEIL = "ceil",
    FLOOR = "floor",
    RAND = "rand",
    NOW = "now",
    YEAR = "year",
    MONTH = "month",
    DAY = "day",
    HOURS = "hours",
    MINUTES = "minutes",
    SECONDS = "seconds",
    TIMEZONE = "timezone",
    TZ = "tz",
    MD5 = "md5",
    SHA1 = "sha1",
    SHA256 = "sha256",
    SHA384 = "sha384",
    SHA512 = "sha512"
}
export declare enum SpecialOperator {
    BOUND = "bound",
    IF = "if",
    COALESCE = "coalesce",
    LOGICAL_OR = "||",
    LOGICAL_AND = "&&",
    SAME_TERM = "sameterm",
    IN = "in",
    NOT_IN = "notin",
    CONCAT = "concat",
    BNODE = "bnode"
}
export declare const RegularOperators: Set<string>;
export declare const SpecialOperators: Set<string>;
export declare const Operators: Set<string>;
export declare enum SetFunction {
    COUNT = "count",
    SUM = "sum",
    MIN = "min",
    MAX = "max",
    AVG = "avg",
    GROUP_CONCAT = "group_concat",
    SAMPLE = "sample"
}
export declare const SetFunctions: Set<SetFunction>;
export declare type NamedOperator = TypeURL.XSD_STRING | TypeURL.XSD_FLOAT | TypeURL.XSD_DOUBLE | TypeURL.XSD_DECIMAL | TypeURL.XSD_INTEGER | TypeURL.XSD_DATE_TIME | TypeURL.XSD_DATE | TypeURL.XSD_BOOLEAN | TypeURL.XSD_TIME | TypeURL.XSD_DURATION | TypeURL.XSD_DAY_TIME_DURATION | TypeURL.XSD_YEAR_MONTH_DURATION;
export declare const NamedOperators: Set<TypeURL>;
